from collections import OrderedDict

def merge_dictionaries(*args):
    if len(args) < 1:
        return {}
    z = args[0].copy()
    for a in args[1:]:
        z.update(a)
    return z

def merge_ordered_dictionaries(*args):
    if len(args) < 1:
        return {}
    z = OrderedDict(args[0].copy())
    for a in args[1:]:
        z.update(a)
    return z


class Map(dict):
    def __init__(self, *args, **kwargs):
        super(Map, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for k, v in arg.items():
                    self[k] = v

        if kwargs:
            for k, v in kwargs.items():
                self[k] = v

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(Map, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(Map, self).__delitem__(key)
        del self.__dict__[key]


import os
import datetime
import codecs
import re
import math

from olm.writer import Writer
from olm.helper import merge_dictionaries

class Index:
    """Represents an index listing"""

    def __init__(self, context):
        self.context = context
        all_files = context.articles
        self.articles = [ article for article in all_files if article.type in (context.ARTICLE_TYPES + context.INDEX_TYPES) ]
        self.template = context.JINJA_ENV.get_template('index.html')
        self.output_filepath = os.path.join(context.OUTPUT_FOLDER, 'index.html')
        self.paginate()

    def paginate(self):
        num_per_page = 10
        num_pages = math.ceil(len(self.articles)/num_per_page)
        pages = []
        articles = self.articles
        for i in range(0, num_pages):
            page_articles = articles[0:num_per_page]
            articles = articles[num_per_page:]
            pages.append(page_articles)
        self.pages = pages

    def paginated_path(self, page_number, basepath):
        if page_number == 1:
            return os.path.join(basepath, 'index.html'), '/'
        return os.path.join(self.context.OUTPUT_FOLDER,'pg', str(page_number) , 'index.html'), os.path.join('pg', str(page_number))


    def write_file(self):
        """Write the article to a file"""
        cached = True
        changes                = self.context['cache_change_types']
        changed_meta           = self.context['cache_changed_meta']
        refresh_triggers       = self.context['INDEX_WRITE_TRIGGERS']
        refresh_meta_triggers  = self.context['INDEX_META_WRITE_TRIGGERS']

        if any(i in changes for i in refresh_triggers):
            cached = False
        if any(any(m in merge_dictionaries(*c) for m in refresh_meta_triggers) for c in changed_meta):
            cached = False
        if not self.context.caching_enabled:
            cached = False
        if cached:
            return False
        
        dirname = os.path.dirname(self.output_filepath)
        for i, page in enumerate(self.pages):
            for article in page:
                if article.summary:
                    article.summary = self.context.MD(article.summary)
            page_number = i + 1
            output_filepath, url = self.paginated_path(page_number, dirname)
            previous_page = None
            next_page = None
            if i > 0: 
                previous_filepath, previous_page = self.paginated_path(page_number - 1, dirname)
            if i < len(self.pages):
                next_filepath, next_page = self.paginated_path(page_number + 1, dirname)
            writer = Writer(
                self.context, 
                output_filepath, 
                self.template,
                index_pages=self.pages, 
                current_page=page,
                current_page_number=page_number,
                previous_page=previous_page,
                next_page=next_page)
            writer.write_file()
        return True


import os
import sys
import imp
from blinker import signal
from olm.logger import get_logger

logger = get_logger('olm.plugins')

class Plugins:
    def __init__(self, context):
        self.receivers = []
        self.load_plugins(context)

    def load_plugins(self, context):
        logger.info('Loading plugins')
        for plugin in context['PLUGINS']:
            logger.debug('Loading plugin %s', plugin)
            sys_path = sys.path
            try:
                path = os.path.join(context.PLUGINS_FOLDER, plugin)
                sys.path.append(path)
                pyfile = os.path.join(path, plugin + '.py')
                py_mod = imp.load_source(plugin, pyfile)
                registrations = getattr(py_mod, 'register')()
                if type(registrations) is tuple:
                    registrations = [ registrations ]
                for registration in registrations:
                    # TODO: Check if valid signal
                    sender = signal(registration[0])
                    receiver = sender.connect(registration[1])
                    self.receivers.append((registration[0], receiver))
                sys.path = sys_path
            except Exception as e:
                sys.path = sys_path
                logger.warn('Plugin %s failed to load.', plugin)
                logger.warn(e)

    def unload_plugins(self):
        for receiver in self.receivers:
            signal(receiver[0]).disconnect(receiver[1])
        


import os
import datetime
import codecs
import re
from urllib.parse import urljoin

from olm.constants import ArticleStatus
from olm.signals import Signal, signals
from olm.helper import merge_dictionaries
from olm.source import Source

class Article(Source):
    """ Represents an article"""
    def __repr__(self):
        return self.output_filepath

    def __init__(self, context, filepath=None, metadata=None, content=None, basename=None):
        if filepath is not None:
            super().__init__(context, filepath=filepath)
        elif metadata is not None and content is not None and basename is not None:
            super().__init__(context, metadata=metadata, content=content, basename=basename)

        self.template   = 'article.html'
        self.date       = datetime.datetime.strptime(self.metadata['date'].strip(), '%Y-%m-%d') if 'date' in self.metadata else datetime.datetime.now()
        self.type       = self.metadata['type'].strip().lower() if 'type' in self.metadata else None
        self.title      = self.metadata['title'] if 'title' in self.metadata else basename
        self.summary    = self.metadata['summary'] if 'summary' in self.metadata else None
        self.location   = self.metadata['location'].strip().lower() if 'location' in self.metadata else None
        
        # Status
        status          = self.metadata['status'].strip().lower() if 'status' in self.metadata else None
        if status == 'unlisted' or self.type == 'unlisted':
            self.status = ArticleStatus.UNLISTED
        elif status == 'draft':
            self.status = ArticleStatus.DRAFT
        else:
            self.status = ArticleStatus.ACTIVE
        
        # Authors
        if 'authors' in self.metadata:
            self.authors = [a.strip() for a in self.metadata['authors'].split(',')]
        elif 'author' in self.metadata:
            self.authors = [a.strip() for a in self.metadata['author'].split(',')]
        else:
            self.authors = []
        for author in self.authors:
            if author in context['authors']:
                context['authors'][author].append(self)
            else:
                context['authors'][author] = [self]

        # Output Filepath
        if self.date and self.location:
            if 'ARTICLE_SLUG' in context:
                slug_dict = merge_dictionaries(vars(self), {'date': self.date.strftime('%Y-%m-%d'), 'location': self.location.lower()})
                output_filename = context.ARTICLE_SLUG.format(**slug_dict)
            else:
                output_filename = '{}-{}.html'.format(self.location.lower(), self.date.strftime('%Y-%m-%d'))
        else:
            output_filename = '{}.html'.format(self.basename)   
        self.output_filepath = os.path.join(context.OUTPUT_FOLDER, 'articles', output_filename)
        self.url             = 'articles/{}'.format(output_filename)

        self.cache_id = self.output_filepath
        self.cache_type = 'ARTICLE'

        signal_sender = Signal(signals.AFTER_ARTICLE_READ)
        signal_sender.send(context=context, article=self)

    def write_file(self, context=None):
        self.context = context if context is not None else self.context
        changes                = self.context['cache_change_types']
        changed_meta           = self.context['cache_changed_meta']
        refresh_triggers       = self.context['ARTICLE_WRITE_TRIGGERS']
        refresh_meta_triggers  = self.context['ARTICLE_META_WRITE_TRIGGERS']
        if any(i in changes for i in refresh_triggers):
            self.same_as_cache = False
        if any(any(m in merge_dictionaries(*c) for m in refresh_meta_triggers) for c in changed_meta):
            self.same_as_cache = False
        super().write_file(context, article=self)
        return not self.same_as_cache


import sys
import os
import time
import argparse

parser = argparse.ArgumentParser(description='Olm static site generator',)
parser.add_argument('src', action="store", help='Path to site folder')
parser.add_argument('-s', '--settings', action="store", default=None, help='Path to settings.py file')
parser.add_argument('-d', '--disable-caching', action="store_true", help='Disable caching')
parser.add_argument('-l', '--log-level', action="store", default="NOTICE", help='Set log level')
args = parser.parse_args()

from olm.logger import get_logger, set_log_level
set_log_level(args.log_level)

# Imports here in order to get correct log level
from olm.context import load_context, load_context_from_file, load_default_context
from olm.plugins import Plugins
from olm.signals import Signal, signals
from olm.website import Site

logger = get_logger('olm')

def main():
    """Main olm function"""
    time_all = time.time()
    logger.notice("Beginning static site generation")

    if args.settings is not None:
        settings_file_path = os.path.abspath(args.settings)
    else:
        settings_file_path = os.path.abspath(os.path.join(args.src, 'settings.py'))
    if os.path.isfile(settings_file_path):
        CONTEXT = load_context_from_file(settings_file_path, load_default_context(args.src))
    else:
        logger.error('No valid settings.py file found')
        sys.exit()
    
    CONTEXT.caching_enabled = True
    if args.disable_caching:
        CONTEXT.caching_enabled = False

    plugins = Plugins(CONTEXT)

    signal_sender = Signal(signals.INITIALISED)
    signal_sender.send(context=CONTEXT)

    site = Site(CONTEXT)
    site.build_site()

    subsites = site.subsites

    for subsite in subsites:
        CONTEXT = load_context_from_file(settings_file_path, load_default_context(args.src))
        CONTEXT.caching_enabled = True
        if args.disable_caching:
            CONTEXT.caching_enabled = False
        plugins.unload_plugins()
        subsite_name = subsite[1:]
        logger.info("")
        logger.info("Found subsite '%s'", subsite_name)
        if subsite_name in CONTEXT.SUBSITES:
            subsite_context = load_context(CONTEXT.SUBSITES[subsite_name], CONTEXT)
        else:
            subsite_context = CONTEXT
        plugins.load_plugins(subsite_context)
        subsite_context.BASE_FOLDER    = os.path.join(CONTEXT.SOURCE_FOLDER, subsite)
        subsite_context.SOURCE_FOLDER  = os.path.join(CONTEXT.SOURCE_FOLDER, subsite)
        subsite_context.OUTPUT_FOLDER  = os.path.join(CONTEXT.OUTPUT_FOLDER, subsite_name)
        subsite_context.CACHE_LOCATION = os.path.join(CONTEXT.BASE_FOLDER, 'cache_' + subsite_name + '.pickle')
        site = Site(subsite_context)
        site.build_site()

    logger.success("Completed everything in %.3f seconds", (time.time() - time_all))

if __name__== "__main__":
  main()


from setuptools import setup

setup(name='olm',
    version='0.0.26',
    description='Static site generator',
    url='https://github.com/rhyst/olm',
    author='Rhys Tyers',
    author_email='',
    license='MIT',
    packages=['olm'],
    zip_safe=False,
    entry_points = {
        'console_scripts': ['olm=olm:main'],
    },
    install_requires=[
        'astroid==1.6.1',
        'blinker==1.4',
        'chardet==3.0.4',
        'coloredlogs==9.0',
        'humanfriendly==4.8',
        'isort==4.2.15',
        'Jinja2==2.10',
        'jsmin==2.2.2',
        'lazy-object-proxy==1.3.1',
        'libsass==0.13.7',
        'MarkupSafe==1.0',
        'mccabe==0.6.1',
        'mistune==0.8.3',
        'pylint==1.8.2',
        'six==1.11.0',
        'verboselogs==1.7',
        'wrapt==1.10.11'
    ]

)
